package com.example.ljgscancode

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.ViewModelProvider
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.example.ljgscancode.databinding.ActivityMainBinding
import com.example.ljgscancode.model.Product
import com.example.ljgscancode.viewmodel.ProductListViewModel
import com.google.android.gms.common.GooglePlayServicesUtil

class MainActivity : AppCompatActivity() {
    private lateinit var mViewBinding: ActivityMainBinding
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        mViewBinding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(mViewBinding.root)

        val model = ViewModelProvider(this).get(ProductListViewModel::class.java)
        model.initData()
        model.getData().observe(this) {
            val rv = mViewBinding.recycleView
            rv.layoutManager = GridLayoutManager(this@MainActivity, 2)
            rv.adapter = ChooseAdapter(it)
        }
    }

    private class ChooseAdapter(val products: List<Product>) :
        RecyclerView.Adapter<ChooseAdapter.ChooseViewHolder>() {
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ChooseViewHolder {
            val view =
                LayoutInflater.from(parent.context).inflate(R.layout.item_choose, parent, false)
            return ChooseViewHolder(view)
        }

        override fun onBindViewHolder(holder: ChooseViewHolder, position: Int) {
            val product = products[position]
            holder.itemView.setOnClickListener { v: View ->
                val intent = Intent(v.context, ScanCodeActivity::class.java)
                intent.putExtra("typeId",product.id )
                v.context.startActivity(intent)
            }
            Glide.with(holder.itemView.context).load(product.imgUrl).into(holder.imageView)
            holder.textView.text = product.name
        }

        override fun getItemCount(): Int = products.size


        class ChooseViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
            var imageView: RatioImageView = itemView.findViewById(R.id.imageView)
            var textView: TextView = itemView.findViewById(R.id.textView)
        }
    }

    override fun onPause() {
        super.onPause()
        Log.d("Lifecycle", "onPause");
    }

    override fun onStop() {
        super.onStop()
        Log.d("Lifecycle", "onStop");
    }

    override fun onDestroy() {
        super.onDestroy()
        Log.d("Lifecycle", "onDestroy");
    }
}