package com.example.ljgscancode.viewmodel

import android.content.Context
import android.widget.Toast
import androidx.core.content.contentValuesOf
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.ljgscancode.ApiService
import com.example.ljgscancode.data.CodeResultDatabase
import kotlinx.coroutines.launch
import java.lang.Exception

class ResultListViewModel : ViewModel() {
    private var mutableLiveData = MutableLiveData<List<String>>()

    private lateinit var typeId: String
    private lateinit var database: CodeResultDatabase

    private val resultList = ArrayList<String>()

    fun getData(): LiveData<List<String>> {
        return mutableLiveData
    }

    fun initData(context: Context, typeId: String) {
        this.typeId = typeId
        try {
            database = CodeResultDatabase.Build(context, 1).build();
            val db = database.writableDatabase

            val cursor = db.rawQuery("select * from CodeResult where type=?", arrayOf(typeId))
            if (cursor.moveToFirst()) {
                do {
                    val code = cursor.getString(cursor.getColumnIndex("code"))
                    resultList.add(code)
                } while (cursor.moveToNext())
            }
            cursor.close()
            mutableLiveData.value = resultList

        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    fun addResult(result: String) {
        if (!resultList.contains(result)) {
            resultList.add(result)
            val db = database.writableDatabase

            db.insert(
                CodeResultDatabase.codeResultTableName,
                null,
                contentValuesOf(
                    "code" to result,
                    "type" to typeId,
                )
            )

            mutableLiveData.value = resultList
        }
    }

    fun intoStock(context: Context) {
        if (resultList.size <= 0) return
        viewModelScope.launch {
            var result = ApiService.instance.apiService.intoStock(1, resultList)

            Toast.makeText(context,result.msg,Toast.LENGTH_SHORT).show()

            if (result.isSuccess) {
                resultList.forEach { deleteCodeFromSQLite(it) }
                resultList.clear()
                mutableLiveData.value = resultList
            }
        }

    }

    private fun deleteCodeFromSQLite(code: String) {
        val db = database.writableDatabase
        db.delete(
            CodeResultDatabase.codeResultTableName,
            "type = ? and code = ?",
            arrayOf(typeId, code),
        )
    }

}